@@ -189,26 +189,7 @@ pub trait Vecf64 {
     /// Linear transformation to [0,1]
     fn lintrans(self) -> Vec<f64>;
     /// Sort vector in a standard way
-    fn sortf(self) -> Vec<f64>;
-
-    /*
-    /// Binary search for insert index I in sorted vector 
-    fn binsearch(self, v: f64) -> usize;
-    /// Merges two ascending sorted vectors
-    fn merge(self, v: &[f64]) -> Vec<f64>;
-    /// Merges two sort indices, returns simply concatenated Vec<f64> and new sort index into it
-    fn merge_immutable(self, idx1: &[usize], v2: &[f64], idx2: &[usize]) -> ( Vec<f64>,Vec<usize> );
-    /// merge indices of two already concatenated sorted vectors
-    fn merge_indices(self, idx1:&[usize], idx2:&[usize]) -> Vec<usize>;
-
-    /// Sorted vector, is wrapper for mergesort below
-    fn sortm(self, ascending:bool) -> Vec<f64>;
-
-    /// Ranking with only n*log(n) complexity, using 'mergesort'
-    fn rank(self, ascending:bool) -> Vec<usize>;
-    /// Immutable merge sort, makes a sort index
-    fn mergesort(self, i:usize, n:usize) -> Vec<usize>;
-    */
+    fn sortf(self) -> Vec<f64>; 
 }
 
 /// Some support for Vec<u8> (vector of bytes)
@@ -294,6 +275,8 @@ pub trait MutVectors {
 pub trait VecVecu8 { 
     /// Centroid = euclidian mean of a set of points  
     fn acentroid(self) -> Vec<f64>;
+    /// Weighted Centre
+    fn wacentroid(self,ws: &[u8]) -> Vec<f64>;
     /// Eccentricity vector added to a non member point,
     fn nxnonmember(self, p:&[f64]) -> Vec<f64>;
     /// Weighted eccentricity vector for a non member point
@@ -310,8 +293,10 @@ pub trait VecVecu8 {
 /// Methods applicable to vector of vectors of <f64>
 pub trait VecVecf64 {
 
-    /// Arithmetic Centroid = euclidian mean of a set of points
+    /// Arithmetic Centre = euclidian mean of a set of points
     fn acentroid(self) -> Vec<f64>;
+    /// Weighted Arithmetic Centre = weighted euclidian mean of a set of points
+    fn wacentroid(self,ws: &[f64]) -> Vec<f64>;
     /// Geometric Centroid
     fn gcentroid(self) -> Vec<f64>;
     /// Harmonic Centroid = harmonic mean of a set of points
@@ -371,6 +356,8 @@ pub trait VecVecf64 {
     fn gmedian(self, eps: f64) -> Vec<f64>; 
     /// The weighted geometric median
     fn wgmedian(self, ws: &[f64],eps: f64) -> Vec<f64>; 
-    /// Lower triangular part of a covariance matrix of a Vec of f64 vectors.
+    /// Flattened lower triangular part of a covariance matrix of a Vec of f64 vectors.
     fn covar(self, med:&[f64]) -> Vec<f64>; 
+    /// Flattened lower triangular part of a covariance matrix for weighted f64 vectors.
+    fn wcovar(self, ws:&[f64], m:&[f64]) -> Vec<f64>;
 }
